# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc4bmi/bmi.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grpc4bmi/bmi.proto',
  package='bmi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x12grpc4bmi/bmi.proto\x12\x03\x62mi\"\x07\n\x05\x45mpty\"(\n\x11InitializeRequest\x12\x13\n\x0b\x63onfig_file\x18\x01 \x01(\t\"(\n\x18GetComponentNameResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\"$\n\x13GetVarNamesResponse\x12\r\n\x05names\x18\x01 \x03(\t\"%\n\x14GetTimeUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"\'\n\x13GetTimeStepResponse\x12\x10\n\x08interval\x18\x01 \x01(\x01\"\x1f\n\x0fGetTimeResponse\x12\x0c\n\x04time\x18\x01 \x01(\x01\"\x1d\n\rGetVarRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"%\n\x12GetVarGridResponse\x12\x0f\n\x07grid_id\x18\x01 \x01(\x05\"\"\n\x12GetVarTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\"&\n\x16GetVarItemSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"$\n\x13GetVarUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"&\n\x14GetVarNBytesResponse\x12\x0e\n\x06nbytes\x18\x01 \x01(\x03\"z\n\x16GetVarLocationResponse\x12\x36\n\x08location\x18\x01 \x01(\x0e\x32$.bmi.GetVarLocationResponse.Location\"(\n\x08Location\x12\x08\n\x04NODE\x10\x00\x12\x08\n\x04\x45\x44GE\x10\x01\x12\x08\n\x04\x46\x41\x43\x45\x10\x02\"%\n\x0fIntArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x03\x42\x02\x10\x01\"\'\n\x11\x46loatArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x02\x42\x02\x10\x01\"(\n\x12\x44oubleArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x01\x42\x02\x10\x01\"\xaa\x01\n\x10GetValueResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"=\n\x18GetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\"\xb3\x01\n\x19GetValueAtIndicesResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\xb7\x01\n\x0fSetValueRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\nvalues_int\x18\x02 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x03 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x04 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"/\n\x12SetValuePtrRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03ref\x18\x02 \x01(\x03\"\xd5\x01\n\x18SetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\x12*\n\nvalues_int\x18\x03 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x04 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x05 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\x1e\n\x0bGridRequest\x12\x0f\n\x07grid_id\x18\x01 \x01(\x03\"#\n\x13GetGridSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"#\n\x13GetGridRankResponse\x12\x0c\n\x04rank\x18\x01 \x01(\x03\"#\n\x13GetGridTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\")\n\x14GetGridShapeResponse\x12\x11\n\x05shape\x18\x01 \x03(\x03\x42\x02\x10\x01\"-\n\x16GetGridSpacingResponse\x12\x13\n\x07spacing\x18\x01 \x03(\x01\x42\x02\x10\x01\"+\n\x15GetGridOriginResponse\x12\x12\n\x06origin\x18\x01 \x03(\x01\x42\x02\x10\x01\"0\n\x15GetGridPointsResponse\x12\x17\n\x0b\x63oordinates\x18\x01 \x03(\x01\x42\x02\x10\x01\"!\n\x10GetCountResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\"2\n\x18GetGridEdgeNodesResponse\x12\x16\n\nedge_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceEdgesResponse\x12\x16\n\nface_edges\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceNodesResponse\x12\x16\n\nface_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"9\n\x1bGetGridNodesPerFaceResponse\x12\x1a\n\x0enodes_per_face\x18\x01 \x03(\x03\x42\x02\x10\x01\x32\x8a\x13\n\nBmiService\x12\x32\n\ninitialize\x12\x16.bmi.InitializeRequest\x1a\n.bmi.Empty\"\x00\x12\"\n\x06update\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12\x31\n\x0bupdateUntil\x12\x14.bmi.GetTimeResponse\x1a\n.bmi.Empty\"\x00\x12$\n\x08\x66inalize\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12?\n\x10getComponentName\x12\n.bmi.Empty\x1a\x1d.bmi.GetComponentNameResponse\"\x00\x12\x38\n\x11getInputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12\x39\n\x12getOutputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12:\n\x10getInputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12;\n\x11getOutputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12\x37\n\x0cgetTimeUnits\x12\n.bmi.Empty\x1a\x19.bmi.GetTimeUnitsResponse\"\x00\x12\x35\n\x0bgetTimeStep\x12\n.bmi.Empty\x1a\x18.bmi.GetTimeStepResponse\"\x00\x12\x34\n\x0egetCurrentTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x32\n\x0cgetStartTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x30\n\ngetEndTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12;\n\ngetVarGrid\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarGridResponse\"\x00\x12;\n\ngetVarType\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarTypeResponse\"\x00\x12\x43\n\x0egetVarItemSize\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarItemSizeResponse\"\x00\x12=\n\x0bgetVarUnits\x12\x12.bmi.GetVarRequest\x1a\x18.bmi.GetVarUnitsResponse\"\x00\x12?\n\x0cgetVarNBytes\x12\x12.bmi.GetVarRequest\x1a\x19.bmi.GetVarNBytesResponse\"\x00\x12\x43\n\x0egetVarLocation\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarLocationResponse\"\x00\x12\x37\n\x08getValue\x12\x12.bmi.GetVarRequest\x1a\x15.bmi.GetValueResponse\"\x00\x12T\n\x11getValueAtIndices\x12\x1d.bmi.GetValueAtIndicesRequest\x1a\x1e.bmi.GetValueAtIndicesResponse\"\x00\x12.\n\x08setValue\x12\x14.bmi.SetValueRequest\x1a\n.bmi.Empty\"\x00\x12@\n\x11setValueAtIndices\x12\x1d.bmi.SetValueAtIndicesRequest\x1a\n.bmi.Empty\"\x00\x12;\n\x0bgetGridSize\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridSizeResponse\"\x00\x12;\n\x0bgetGridType\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridTypeResponse\"\x00\x12;\n\x0bgetGridRank\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridRankResponse\"\x00\x12=\n\x0cgetGridShape\x12\x10.bmi.GridRequest\x1a\x19.bmi.GetGridShapeResponse\"\x00\x12\x41\n\x0egetGridSpacing\x12\x10.bmi.GridRequest\x1a\x1b.bmi.GetGridSpacingResponse\"\x00\x12?\n\rgetGridOrigin\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridOriginResponse\"\x00\x12:\n\x08getGridX\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridY\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridZ\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12=\n\x10getGridNodeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridEdgeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridFaceCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12\x45\n\x10getGridEdgeNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridEdgeNodesResponse\"\x00\x12\x45\n\x10getGridFaceNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceNodesResponse\"\x00\x12\x45\n\x10getGridFaceEdges\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceEdgesResponse\"\x00\x12K\n\x13getGridNodesPerFace\x12\x10.bmi.GridRequest\x1a .bmi.GetGridNodesPerFaceResponse\"\x00\x62\x06proto3'
)



_GETVARLOCATIONRESPONSE_LOCATION = _descriptor.EnumDescriptor(
  name='Location',
  full_name='bmi.GetVarLocationResponse.Location',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NODE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EDGE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FACE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=577,
  serialized_end=617,
)
_sym_db.RegisterEnumDescriptor(_GETVARLOCATIONRESPONSE_LOCATION)


_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='bmi.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27,
  serialized_end=34,
)


_INITIALIZEREQUEST = _descriptor.Descriptor(
  name='InitializeRequest',
  full_name='bmi.InitializeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config_file', full_name='bmi.InitializeRequest.config_file', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=76,
)


_GETCOMPONENTNAMERESPONSE = _descriptor.Descriptor(
  name='GetComponentNameResponse',
  full_name='bmi.GetComponentNameResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetComponentNameResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=118,
)


_GETVARNAMESRESPONSE = _descriptor.Descriptor(
  name='GetVarNamesResponse',
  full_name='bmi.GetVarNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='bmi.GetVarNamesResponse.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=156,
)


_GETTIMEUNITSRESPONSE = _descriptor.Descriptor(
  name='GetTimeUnitsResponse',
  full_name='bmi.GetTimeUnitsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='units', full_name='bmi.GetTimeUnitsResponse.units', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=158,
  serialized_end=195,
)


_GETTIMESTEPRESPONSE = _descriptor.Descriptor(
  name='GetTimeStepResponse',
  full_name='bmi.GetTimeStepResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interval', full_name='bmi.GetTimeStepResponse.interval', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=236,
)


_GETTIMERESPONSE = _descriptor.Descriptor(
  name='GetTimeResponse',
  full_name='bmi.GetTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='bmi.GetTimeResponse.time', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=238,
  serialized_end=269,
)


_GETVARREQUEST = _descriptor.Descriptor(
  name='GetVarRequest',
  full_name='bmi.GetVarRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetVarRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=271,
  serialized_end=300,
)


_GETVARGRIDRESPONSE = _descriptor.Descriptor(
  name='GetVarGridResponse',
  full_name='bmi.GetVarGridResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grid_id', full_name='bmi.GetVarGridResponse.grid_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=339,
)


_GETVARTYPERESPONSE = _descriptor.Descriptor(
  name='GetVarTypeResponse',
  full_name='bmi.GetVarTypeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bmi.GetVarTypeResponse.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=341,
  serialized_end=375,
)


_GETVARITEMSIZERESPONSE = _descriptor.Descriptor(
  name='GetVarItemSizeResponse',
  full_name='bmi.GetVarItemSizeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='bmi.GetVarItemSizeResponse.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=377,
  serialized_end=415,
)


_GETVARUNITSRESPONSE = _descriptor.Descriptor(
  name='GetVarUnitsResponse',
  full_name='bmi.GetVarUnitsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='units', full_name='bmi.GetVarUnitsResponse.units', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=417,
  serialized_end=453,
)


_GETVARNBYTESRESPONSE = _descriptor.Descriptor(
  name='GetVarNBytesResponse',
  full_name='bmi.GetVarNBytesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nbytes', full_name='bmi.GetVarNBytesResponse.nbytes', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=455,
  serialized_end=493,
)


_GETVARLOCATIONRESPONSE = _descriptor.Descriptor(
  name='GetVarLocationResponse',
  full_name='bmi.GetVarLocationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='bmi.GetVarLocationResponse.location', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETVARLOCATIONRESPONSE_LOCATION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=495,
  serialized_end=617,
)


_INTARRAYMESSAGE = _descriptor.Descriptor(
  name='IntArrayMessage',
  full_name='bmi.IntArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.IntArrayMessage.values', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=619,
  serialized_end=656,
)


_FLOATARRAYMESSAGE = _descriptor.Descriptor(
  name='FloatArrayMessage',
  full_name='bmi.FloatArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.FloatArrayMessage.values', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=697,
)


_DOUBLEARRAYMESSAGE = _descriptor.Descriptor(
  name='DoubleArrayMessage',
  full_name='bmi.DoubleArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.DoubleArrayMessage.values', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=699,
  serialized_end=739,
)


_GETVALUERESPONSE = _descriptor.Descriptor(
  name='GetValueResponse',
  full_name='bmi.GetValueResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.GetValueResponse.values_int', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.GetValueResponse.values_float', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.GetValueResponse.values_double', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.GetValueResponse.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=742,
  serialized_end=912,
)


_GETVALUEATINDICESREQUEST = _descriptor.Descriptor(
  name='GetValueAtIndicesRequest',
  full_name='bmi.GetValueAtIndicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetValueAtIndicesRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indices', full_name='bmi.GetValueAtIndicesRequest.indices', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=914,
  serialized_end=975,
)


_GETVALUEATINDICESRESPONSE = _descriptor.Descriptor(
  name='GetValueAtIndicesResponse',
  full_name='bmi.GetValueAtIndicesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.GetValueAtIndicesResponse.values_int', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.GetValueAtIndicesResponse.values_float', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.GetValueAtIndicesResponse.values_double', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.GetValueAtIndicesResponse.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=978,
  serialized_end=1157,
)


_SETVALUEREQUEST = _descriptor.Descriptor(
  name='SetValueRequest',
  full_name='bmi.SetValueRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValueRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.SetValueRequest.values_int', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.SetValueRequest.values_float', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.SetValueRequest.values_double', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.SetValueRequest.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1160,
  serialized_end=1343,
)


_SETVALUEPTRREQUEST = _descriptor.Descriptor(
  name='SetValuePtrRequest',
  full_name='bmi.SetValuePtrRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValuePtrRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ref', full_name='bmi.SetValuePtrRequest.ref', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1345,
  serialized_end=1392,
)


_SETVALUEATINDICESREQUEST = _descriptor.Descriptor(
  name='SetValueAtIndicesRequest',
  full_name='bmi.SetValueAtIndicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValueAtIndicesRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indices', full_name='bmi.SetValueAtIndicesRequest.indices', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.SetValueAtIndicesRequest.values_int', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.SetValueAtIndicesRequest.values_float', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.SetValueAtIndicesRequest.values_double', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.SetValueAtIndicesRequest.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1395,
  serialized_end=1608,
)


_GRIDREQUEST = _descriptor.Descriptor(
  name='GridRequest',
  full_name='bmi.GridRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grid_id', full_name='bmi.GridRequest.grid_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1610,
  serialized_end=1640,
)


_GETGRIDSIZERESPONSE = _descriptor.Descriptor(
  name='GetGridSizeResponse',
  full_name='bmi.GetGridSizeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='bmi.GetGridSizeResponse.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1642,
  serialized_end=1677,
)


_GETGRIDRANKRESPONSE = _descriptor.Descriptor(
  name='GetGridRankResponse',
  full_name='bmi.GetGridRankResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rank', full_name='bmi.GetGridRankResponse.rank', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1679,
  serialized_end=1714,
)


_GETGRIDTYPERESPONSE = _descriptor.Descriptor(
  name='GetGridTypeResponse',
  full_name='bmi.GetGridTypeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bmi.GetGridTypeResponse.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1716,
  serialized_end=1751,
)


_GETGRIDSHAPERESPONSE = _descriptor.Descriptor(
  name='GetGridShapeResponse',
  full_name='bmi.GetGridShapeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='shape', full_name='bmi.GetGridShapeResponse.shape', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1753,
  serialized_end=1794,
)


_GETGRIDSPACINGRESPONSE = _descriptor.Descriptor(
  name='GetGridSpacingResponse',
  full_name='bmi.GetGridSpacingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='spacing', full_name='bmi.GetGridSpacingResponse.spacing', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1796,
  serialized_end=1841,
)


_GETGRIDORIGINRESPONSE = _descriptor.Descriptor(
  name='GetGridOriginResponse',
  full_name='bmi.GetGridOriginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='origin', full_name='bmi.GetGridOriginResponse.origin', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1843,
  serialized_end=1886,
)


_GETGRIDPOINTSRESPONSE = _descriptor.Descriptor(
  name='GetGridPointsResponse',
  full_name='bmi.GetGridPointsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='bmi.GetGridPointsResponse.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1888,
  serialized_end=1936,
)


_GETCOUNTRESPONSE = _descriptor.Descriptor(
  name='GetCountResponse',
  full_name='bmi.GetCountResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count', full_name='bmi.GetCountResponse.count', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1938,
  serialized_end=1971,
)


_GETGRIDEDGENODESRESPONSE = _descriptor.Descriptor(
  name='GetGridEdgeNodesResponse',
  full_name='bmi.GetGridEdgeNodesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='edge_nodes', full_name='bmi.GetGridEdgeNodesResponse.edge_nodes', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1973,
  serialized_end=2023,
)


_GETGRIDFACEEDGESRESPONSE = _descriptor.Descriptor(
  name='GetGridFaceEdgesResponse',
  full_name='bmi.GetGridFaceEdgesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='face_edges', full_name='bmi.GetGridFaceEdgesResponse.face_edges', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2025,
  serialized_end=2075,
)


_GETGRIDFACENODESRESPONSE = _descriptor.Descriptor(
  name='GetGridFaceNodesResponse',
  full_name='bmi.GetGridFaceNodesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='face_nodes', full_name='bmi.GetGridFaceNodesResponse.face_nodes', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2077,
  serialized_end=2127,
)


_GETGRIDNODESPERFACERESPONSE = _descriptor.Descriptor(
  name='GetGridNodesPerFaceResponse',
  full_name='bmi.GetGridNodesPerFaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes_per_face', full_name='bmi.GetGridNodesPerFaceResponse.nodes_per_face', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2129,
  serialized_end=2186,
)

_GETVARLOCATIONRESPONSE.fields_by_name['location'].enum_type = _GETVARLOCATIONRESPONSE_LOCATION
_GETVARLOCATIONRESPONSE_LOCATION.containing_type = _GETVARLOCATIONRESPONSE
_GETVALUERESPONSE.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_GETVALUERESPONSE.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_GETVALUERESPONSE.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_int'])
_GETVALUERESPONSE.fields_by_name['values_int'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_float'])
_GETVALUERESPONSE.fields_by_name['values_float'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_double'])
_GETVALUERESPONSE.fields_by_name['values_double'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_int'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_int'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_float'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_float'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_double'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_double'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_SETVALUEREQUEST.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_SETVALUEREQUEST.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_SETVALUEREQUEST.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_int'])
_SETVALUEREQUEST.fields_by_name['values_int'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_float'])
_SETVALUEREQUEST.fields_by_name['values_float'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_double'])
_SETVALUEREQUEST.fields_by_name['values_double'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_SETVALUEATINDICESREQUEST.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_SETVALUEATINDICESREQUEST.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_int'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_int'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_float'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_float'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_double'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_double'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['InitializeRequest'] = _INITIALIZEREQUEST
DESCRIPTOR.message_types_by_name['GetComponentNameResponse'] = _GETCOMPONENTNAMERESPONSE
DESCRIPTOR.message_types_by_name['GetVarNamesResponse'] = _GETVARNAMESRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeUnitsResponse'] = _GETTIMEUNITSRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeStepResponse'] = _GETTIMESTEPRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeResponse'] = _GETTIMERESPONSE
DESCRIPTOR.message_types_by_name['GetVarRequest'] = _GETVARREQUEST
DESCRIPTOR.message_types_by_name['GetVarGridResponse'] = _GETVARGRIDRESPONSE
DESCRIPTOR.message_types_by_name['GetVarTypeResponse'] = _GETVARTYPERESPONSE
DESCRIPTOR.message_types_by_name['GetVarItemSizeResponse'] = _GETVARITEMSIZERESPONSE
DESCRIPTOR.message_types_by_name['GetVarUnitsResponse'] = _GETVARUNITSRESPONSE
DESCRIPTOR.message_types_by_name['GetVarNBytesResponse'] = _GETVARNBYTESRESPONSE
DESCRIPTOR.message_types_by_name['GetVarLocationResponse'] = _GETVARLOCATIONRESPONSE
DESCRIPTOR.message_types_by_name['IntArrayMessage'] = _INTARRAYMESSAGE
DESCRIPTOR.message_types_by_name['FloatArrayMessage'] = _FLOATARRAYMESSAGE
DESCRIPTOR.message_types_by_name['DoubleArrayMessage'] = _DOUBLEARRAYMESSAGE
DESCRIPTOR.message_types_by_name['GetValueResponse'] = _GETVALUERESPONSE
DESCRIPTOR.message_types_by_name['GetValueAtIndicesRequest'] = _GETVALUEATINDICESREQUEST
DESCRIPTOR.message_types_by_name['GetValueAtIndicesResponse'] = _GETVALUEATINDICESRESPONSE
DESCRIPTOR.message_types_by_name['SetValueRequest'] = _SETVALUEREQUEST
DESCRIPTOR.message_types_by_name['SetValuePtrRequest'] = _SETVALUEPTRREQUEST
DESCRIPTOR.message_types_by_name['SetValueAtIndicesRequest'] = _SETVALUEATINDICESREQUEST
DESCRIPTOR.message_types_by_name['GridRequest'] = _GRIDREQUEST
DESCRIPTOR.message_types_by_name['GetGridSizeResponse'] = _GETGRIDSIZERESPONSE
DESCRIPTOR.message_types_by_name['GetGridRankResponse'] = _GETGRIDRANKRESPONSE
DESCRIPTOR.message_types_by_name['GetGridTypeResponse'] = _GETGRIDTYPERESPONSE
DESCRIPTOR.message_types_by_name['GetGridShapeResponse'] = _GETGRIDSHAPERESPONSE
DESCRIPTOR.message_types_by_name['GetGridSpacingResponse'] = _GETGRIDSPACINGRESPONSE
DESCRIPTOR.message_types_by_name['GetGridOriginResponse'] = _GETGRIDORIGINRESPONSE
DESCRIPTOR.message_types_by_name['GetGridPointsResponse'] = _GETGRIDPOINTSRESPONSE
DESCRIPTOR.message_types_by_name['GetCountResponse'] = _GETCOUNTRESPONSE
DESCRIPTOR.message_types_by_name['GetGridEdgeNodesResponse'] = _GETGRIDEDGENODESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridFaceEdgesResponse'] = _GETGRIDFACEEDGESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridFaceNodesResponse'] = _GETGRIDFACENODESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridNodesPerFaceResponse'] = _GETGRIDNODESPERFACERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.Empty)
  })
_sym_db.RegisterMessage(Empty)

InitializeRequest = _reflection.GeneratedProtocolMessageType('InitializeRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZEREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.InitializeRequest)
  })
_sym_db.RegisterMessage(InitializeRequest)

GetComponentNameResponse = _reflection.GeneratedProtocolMessageType('GetComponentNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCOMPONENTNAMERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetComponentNameResponse)
  })
_sym_db.RegisterMessage(GetComponentNameResponse)

GetVarNamesResponse = _reflection.GeneratedProtocolMessageType('GetVarNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARNAMESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarNamesResponse)
  })
_sym_db.RegisterMessage(GetVarNamesResponse)

GetTimeUnitsResponse = _reflection.GeneratedProtocolMessageType('GetTimeUnitsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMEUNITSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeUnitsResponse)
  })
_sym_db.RegisterMessage(GetTimeUnitsResponse)

GetTimeStepResponse = _reflection.GeneratedProtocolMessageType('GetTimeStepResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMESTEPRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeStepResponse)
  })
_sym_db.RegisterMessage(GetTimeStepResponse)

GetTimeResponse = _reflection.GeneratedProtocolMessageType('GetTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeResponse)
  })
_sym_db.RegisterMessage(GetTimeResponse)

GetVarRequest = _reflection.GeneratedProtocolMessageType('GetVarRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVARREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarRequest)
  })
_sym_db.RegisterMessage(GetVarRequest)

GetVarGridResponse = _reflection.GeneratedProtocolMessageType('GetVarGridResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARGRIDRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarGridResponse)
  })
_sym_db.RegisterMessage(GetVarGridResponse)

GetVarTypeResponse = _reflection.GeneratedProtocolMessageType('GetVarTypeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARTYPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarTypeResponse)
  })
_sym_db.RegisterMessage(GetVarTypeResponse)

GetVarItemSizeResponse = _reflection.GeneratedProtocolMessageType('GetVarItemSizeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARITEMSIZERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarItemSizeResponse)
  })
_sym_db.RegisterMessage(GetVarItemSizeResponse)

GetVarUnitsResponse = _reflection.GeneratedProtocolMessageType('GetVarUnitsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARUNITSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarUnitsResponse)
  })
_sym_db.RegisterMessage(GetVarUnitsResponse)

GetVarNBytesResponse = _reflection.GeneratedProtocolMessageType('GetVarNBytesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARNBYTESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarNBytesResponse)
  })
_sym_db.RegisterMessage(GetVarNBytesResponse)

GetVarLocationResponse = _reflection.GeneratedProtocolMessageType('GetVarLocationResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARLOCATIONRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarLocationResponse)
  })
_sym_db.RegisterMessage(GetVarLocationResponse)

IntArrayMessage = _reflection.GeneratedProtocolMessageType('IntArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _INTARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.IntArrayMessage)
  })
_sym_db.RegisterMessage(IntArrayMessage)

FloatArrayMessage = _reflection.GeneratedProtocolMessageType('FloatArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _FLOATARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.FloatArrayMessage)
  })
_sym_db.RegisterMessage(FloatArrayMessage)

DoubleArrayMessage = _reflection.GeneratedProtocolMessageType('DoubleArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _DOUBLEARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.DoubleArrayMessage)
  })
_sym_db.RegisterMessage(DoubleArrayMessage)

GetValueResponse = _reflection.GeneratedProtocolMessageType('GetValueResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueResponse)
  })
_sym_db.RegisterMessage(GetValueResponse)

GetValueAtIndicesRequest = _reflection.GeneratedProtocolMessageType('GetValueAtIndicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUEATINDICESREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueAtIndicesRequest)
  })
_sym_db.RegisterMessage(GetValueAtIndicesRequest)

GetValueAtIndicesResponse = _reflection.GeneratedProtocolMessageType('GetValueAtIndicesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUEATINDICESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueAtIndicesResponse)
  })
_sym_db.RegisterMessage(GetValueAtIndicesResponse)

SetValueRequest = _reflection.GeneratedProtocolMessageType('SetValueRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValueRequest)
  })
_sym_db.RegisterMessage(SetValueRequest)

SetValuePtrRequest = _reflection.GeneratedProtocolMessageType('SetValuePtrRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEPTRREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValuePtrRequest)
  })
_sym_db.RegisterMessage(SetValuePtrRequest)

SetValueAtIndicesRequest = _reflection.GeneratedProtocolMessageType('SetValueAtIndicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEATINDICESREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValueAtIndicesRequest)
  })
_sym_db.RegisterMessage(SetValueAtIndicesRequest)

GridRequest = _reflection.GeneratedProtocolMessageType('GridRequest', (_message.Message,), {
  'DESCRIPTOR' : _GRIDREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GridRequest)
  })
_sym_db.RegisterMessage(GridRequest)

GetGridSizeResponse = _reflection.GeneratedProtocolMessageType('GetGridSizeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSIZERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridSizeResponse)
  })
_sym_db.RegisterMessage(GetGridSizeResponse)

GetGridRankResponse = _reflection.GeneratedProtocolMessageType('GetGridRankResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDRANKRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridRankResponse)
  })
_sym_db.RegisterMessage(GetGridRankResponse)

GetGridTypeResponse = _reflection.GeneratedProtocolMessageType('GetGridTypeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDTYPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridTypeResponse)
  })
_sym_db.RegisterMessage(GetGridTypeResponse)

GetGridShapeResponse = _reflection.GeneratedProtocolMessageType('GetGridShapeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSHAPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridShapeResponse)
  })
_sym_db.RegisterMessage(GetGridShapeResponse)

GetGridSpacingResponse = _reflection.GeneratedProtocolMessageType('GetGridSpacingResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSPACINGRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridSpacingResponse)
  })
_sym_db.RegisterMessage(GetGridSpacingResponse)

GetGridOriginResponse = _reflection.GeneratedProtocolMessageType('GetGridOriginResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDORIGINRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridOriginResponse)
  })
_sym_db.RegisterMessage(GetGridOriginResponse)

GetGridPointsResponse = _reflection.GeneratedProtocolMessageType('GetGridPointsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDPOINTSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridPointsResponse)
  })
_sym_db.RegisterMessage(GetGridPointsResponse)

GetCountResponse = _reflection.GeneratedProtocolMessageType('GetCountResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCOUNTRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetCountResponse)
  })
_sym_db.RegisterMessage(GetCountResponse)

GetGridEdgeNodesResponse = _reflection.GeneratedProtocolMessageType('GetGridEdgeNodesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDEDGENODESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridEdgeNodesResponse)
  })
_sym_db.RegisterMessage(GetGridEdgeNodesResponse)

GetGridFaceEdgesResponse = _reflection.GeneratedProtocolMessageType('GetGridFaceEdgesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDFACEEDGESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridFaceEdgesResponse)
  })
_sym_db.RegisterMessage(GetGridFaceEdgesResponse)

GetGridFaceNodesResponse = _reflection.GeneratedProtocolMessageType('GetGridFaceNodesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDFACENODESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridFaceNodesResponse)
  })
_sym_db.RegisterMessage(GetGridFaceNodesResponse)

GetGridNodesPerFaceResponse = _reflection.GeneratedProtocolMessageType('GetGridNodesPerFaceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDNODESPERFACERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridNodesPerFaceResponse)
  })
_sym_db.RegisterMessage(GetGridNodesPerFaceResponse)


_INTARRAYMESSAGE.fields_by_name['values']._options = None
_FLOATARRAYMESSAGE.fields_by_name['values']._options = None
_DOUBLEARRAYMESSAGE.fields_by_name['values']._options = None
_GETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
_SETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
_GETGRIDSHAPERESPONSE.fields_by_name['shape']._options = None
_GETGRIDSPACINGRESPONSE.fields_by_name['spacing']._options = None
_GETGRIDORIGINRESPONSE.fields_by_name['origin']._options = None
_GETGRIDPOINTSRESPONSE.fields_by_name['coordinates']._options = None
_GETGRIDEDGENODESRESPONSE.fields_by_name['edge_nodes']._options = None
_GETGRIDFACEEDGESRESPONSE.fields_by_name['face_edges']._options = None
_GETGRIDFACENODESRESPONSE.fields_by_name['face_nodes']._options = None
_GETGRIDNODESPERFACERESPONSE.fields_by_name['nodes_per_face']._options = None

_BMISERVICE = _descriptor.ServiceDescriptor(
  name='BmiService',
  full_name='bmi.BmiService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2189,
  serialized_end=4631,
  methods=[
  _descriptor.MethodDescriptor(
    name='initialize',
    full_name='bmi.BmiService.initialize',
    index=0,
    containing_service=None,
    input_type=_INITIALIZEREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='update',
    full_name='bmi.BmiService.update',
    index=1,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='updateUntil',
    full_name='bmi.BmiService.updateUntil',
    index=2,
    containing_service=None,
    input_type=_GETTIMERESPONSE,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='finalize',
    full_name='bmi.BmiService.finalize',
    index=3,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getComponentName',
    full_name='bmi.BmiService.getComponentName',
    index=4,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOMPONENTNAMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInputItemCount',
    full_name='bmi.BmiService.getInputItemCount',
    index=5,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOutputItemCount',
    full_name='bmi.BmiService.getOutputItemCount',
    index=6,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInputVarNames',
    full_name='bmi.BmiService.getInputVarNames',
    index=7,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETVARNAMESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOutputVarNames',
    full_name='bmi.BmiService.getOutputVarNames',
    index=8,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETVARNAMESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTimeUnits',
    full_name='bmi.BmiService.getTimeUnits',
    index=9,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMEUNITSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTimeStep',
    full_name='bmi.BmiService.getTimeStep',
    index=10,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMESTEPRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getCurrentTime',
    full_name='bmi.BmiService.getCurrentTime',
    index=11,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getStartTime',
    full_name='bmi.BmiService.getStartTime',
    index=12,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getEndTime',
    full_name='bmi.BmiService.getEndTime',
    index=13,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarGrid',
    full_name='bmi.BmiService.getVarGrid',
    index=14,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARGRIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarType',
    full_name='bmi.BmiService.getVarType',
    index=15,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARTYPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarItemSize',
    full_name='bmi.BmiService.getVarItemSize',
    index=16,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARITEMSIZERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarUnits',
    full_name='bmi.BmiService.getVarUnits',
    index=17,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARUNITSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarNBytes',
    full_name='bmi.BmiService.getVarNBytes',
    index=18,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARNBYTESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarLocation',
    full_name='bmi.BmiService.getVarLocation',
    index=19,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARLOCATIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getValue',
    full_name='bmi.BmiService.getValue',
    index=20,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVALUERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getValueAtIndices',
    full_name='bmi.BmiService.getValueAtIndices',
    index=21,
    containing_service=None,
    input_type=_GETVALUEATINDICESREQUEST,
    output_type=_GETVALUEATINDICESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setValue',
    full_name='bmi.BmiService.setValue',
    index=22,
    containing_service=None,
    input_type=_SETVALUEREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setValueAtIndices',
    full_name='bmi.BmiService.setValueAtIndices',
    index=23,
    containing_service=None,
    input_type=_SETVALUEATINDICESREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridSize',
    full_name='bmi.BmiService.getGridSize',
    index=24,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSIZERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridType',
    full_name='bmi.BmiService.getGridType',
    index=25,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDTYPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridRank',
    full_name='bmi.BmiService.getGridRank',
    index=26,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDRANKRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridShape',
    full_name='bmi.BmiService.getGridShape',
    index=27,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSHAPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridSpacing',
    full_name='bmi.BmiService.getGridSpacing',
    index=28,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSPACINGRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridOrigin',
    full_name='bmi.BmiService.getGridOrigin',
    index=29,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDORIGINRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridX',
    full_name='bmi.BmiService.getGridX',
    index=30,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridY',
    full_name='bmi.BmiService.getGridY',
    index=31,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridZ',
    full_name='bmi.BmiService.getGridZ',
    index=32,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridNodeCount',
    full_name='bmi.BmiService.getGridNodeCount',
    index=33,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridEdgeCount',
    full_name='bmi.BmiService.getGridEdgeCount',
    index=34,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceCount',
    full_name='bmi.BmiService.getGridFaceCount',
    index=35,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridEdgeNodes',
    full_name='bmi.BmiService.getGridEdgeNodes',
    index=36,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDEDGENODESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceNodes',
    full_name='bmi.BmiService.getGridFaceNodes',
    index=37,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDFACENODESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceEdges',
    full_name='bmi.BmiService.getGridFaceEdges',
    index=38,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDFACEEDGESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridNodesPerFace',
    full_name='bmi.BmiService.getGridNodesPerFace',
    index=39,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDNODESPERFACERESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_BMISERVICE)

DESCRIPTOR.services_by_name['BmiService'] = _BMISERVICE

# @@protoc_insertion_point(module_scope)



from io import BytesIO

import docker
import pytest

from grpc4bmi.bmi_client_docker import BmiClientDocker, DeadDockerContainerException
from grpc4bmi.reserve import reserve_grid_padding, reserve_values

walrus_docker_image = 'ewatercycle/walrus-grpc4bmi:v0.3.1'


@pytest.fixture()
def walrus_model(tmp_path, walrus_input):
    model = BmiClientDocker(image=walrus_docker_image, image_port=55555, input_dir=str(tmp_path))
    yield model
    del model


@pytest.fixture()
def walrus_model_with_extra_volume(tmp_path, walrus_input_on_extra_volume):
    (input_dir, extra_volumes) = walrus_input_on_extra_volume
    model = BmiClientDocker(image=walrus_docker_image,
                            image_port=55555,
                            input_dir=str(input_dir),
                            extra_volumes=extra_volumes)
    yield model
    del model


@pytest.fixture()
def exit_container():
    client = docker.from_env()
    prog = "import sys;print('my stderr log message', file=sys.stderr);print('my stdout log message');sys.exit(25)"
    dockerfile = BytesIO(f'FROM {walrus_docker_image}\nCMD ["python3", "-c", "{prog}" ]\n'.encode())
    image, logs = client.images.build(fileobj=dockerfile)
    yield image.id
    client.images.remove(image.id, force=True)


class TestBmiClientDocker:
    def test_component_name(self, walrus_model):
        assert walrus_model.get_component_name() == 'WALRUS'

    def test_get_grid_origin(self, walrus_input, walrus_model):
        walrus_model.initialize(str(walrus_input))
        grid_id = walrus_model.get_var_grid('Q')

        result = reserve_grid_padding(walrus_model, grid_id)
        assert len(walrus_model.get_grid_origin(grid_id, result)) == 2

    def test_initialize(self, walrus_input, walrus_model):
        walrus_model.initialize(str(walrus_input))
        assert walrus_model.get_current_time() == walrus_model.get_start_time()

    def test_get_value_ptr(self, walrus_model):
        with pytest.raises(NotImplementedError):
            walrus_model.get_value_ptr('Q')

    def test_extra_volume(self, walrus_model_with_extra_volume):
        walrus_model_with_extra_volume.initialize('/data/input/config.yml')
        walrus_model_with_extra_volume.update()

        # After initialization and update the forcings have been read from the extra volume
        result = reserve_values(walrus_model_with_extra_volume, 'Q')
        assert len(walrus_model_with_extra_volume.get_value('Q', result)) == 1

    def test_get_var_location(self, walrus_model):
        assert walrus_model.get_var_location('Q') == 'node'

    def test_logs(self, walrus_model):
        lg = walrus_model.logs()
        assert b"Loading required package:" in lg

    def test_inputdir_absent(self, tmp_path):
        dirthatdoesnotexist = 'dirthatdoesnotexist'
        input_dir = tmp_path / dirthatdoesnotexist
        with pytest.raises(NotADirectoryError, match=dirthatdoesnotexist):
            BmiClientDocker(image=walrus_docker_image, image_port=55555, input_dir=str(input_dir))

    def test_container_start_failure(self, exit_container):
        expected = r"Failed to start Docker container with image"
        with pytest.raises(DeadDockerContainerException, match=expected) as excinfo:
            BmiClientDocker(image=exit_container)

        assert excinfo.value.exitcode == 25
        assert b'my stderr' in excinfo.value.logs
        assert b'my stdout' in excinfo.value.logs



import logging
from unittest.mock import Mock

import numpy
import numpy.random
import pytest
import grpc
from google.rpc import error_details_pb2, status_pb2

from grpc4bmi import bmi_pb2
from grpc4bmi.bmi_grpc_server import BmiServer
from grpc4bmi.reserve import reserve_values, reserve_grid_shape, reserve_grid_padding
from test.fake_models import SomeException, FailingModel, Rect3DGridModel, UnstructuredGridBmiModel
from test.flatbmiheat import FlatBmiHeat

"""
Unit tests for the BMI server class. Every test performs cross-checking with a local instance of the BMI heat toy model.
"""

logging.basicConfig(level=logging.DEBUG)


class RequestStub(object):
    def __init__(self):
        self.config_file = ""

    def HasField(self, name):
        return hasattr(self, name)


class value_wrapper(object):
    def __init__(self, vals):
        self.values = vals.flatten()


def make_string(obj):
    return '' if obj is None else str(obj)


def make_list(obj):
    if obj is list:
        return obj
    if obj is None:
        return []
    return [obj]


def make_bmi_classes(init=False):
    server, local = BmiServer(FlatBmiHeat()), FlatBmiHeat()
    if init:
        req = RequestStub()
        numpy.random.seed(0)
        server.initialize(req, None)
        numpy.random.seed(0)
        local.initialize(None)
    return server, local


def make_var_request(local):
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    setattr(request, "name", varname)
    return request, varname


def make_grid_request(local):
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    grid_id = local.get_var_grid(varname)
    setattr(request, "grid_id", grid_id)
    return grid_id, request


def test_server_start():
    server, local = make_bmi_classes()
    assert server is not None
    del server


def test_component_name():
    server, local = make_bmi_classes()
    assert server.getComponentName(None, None).name == local.get_component_name()
    del server


def test_input_var_names():
    server, local = make_bmi_classes()
    assert server.getInputVarNames(None, None).names == list(local.get_input_var_names())
    del server


def test_output_var_names():
    server, local = make_bmi_classes()
    assert server.getOutputVarNames(None, None).names == list(local.get_output_var_names())
    del server


def test_initialize():
    server, local = make_bmi_classes(True)
    assert server is not None
    server.finalize(None, None)
    del server


def test_update():
    server, local = make_bmi_classes(True)
    server.update(None, None)
    assert server is not None
    server.finalize(None, None)
    del server


def test_get_time_unit():
    server, local = make_bmi_classes()
    assert server.getTimeUnits(None, None).units == make_string(local.get_time_units())
    server.finalize(None, None)
    del server


def test_get_time_step():
    server, local = make_bmi_classes(True)
    assert server.getTimeStep(None, None).interval == local.get_time_step()
    server.finalize(None, None)
    del server


def test_get_current_time():
    server, local = make_bmi_classes(True)
    assert server.getCurrentTime(None, None).time == local.get_current_time()
    server.finalize(None, None)
    del server


def test_get_updated_time():
    server, local = make_bmi_classes(True)
    server.update(None, None)
    assert server.getCurrentTime(None, None).time != local.get_current_time()
    local.update()
    assert server.getCurrentTime(None, None).time == local.get_current_time()
    server.finalize(None, None)
    del server


def test_get_start_end_time():
    server, local = make_bmi_classes(True)
    assert server.getStartTime(None, None).time == local.get_start_time()
    assert server.getEndTime(None, None).time == local.get_end_time()
    server.finalize(None, None)
    del server


def test_get_var_grid():
    server, local = make_bmi_classes(True)
    request, varname = make_var_request(local)

    assert server.getVarGrid(request, None).grid_id == local.get_var_grid(varname)
    del server


def test_get_var_type():
    server, local = make_bmi_classes(True)
    request, varname = make_var_request(local)

    assert server.getVarType(request, None).type == local.get_var_type(varname)
    del server


def test_get_var_units():
    server, local = make_bmi_classes(True)
    request, varname = make_var_request(local)

    assert server.getVarUnits(request, None).units == local.get_var_units(varname)
    del server


def test_get_var_nbytes():
    server, local = make_bmi_classes(True)
    request, varname = make_var_request(local)

    assert server.getVarNBytes(request, None).nbytes == local.get_var_nbytes(varname)
    del server


def test_get_var_location():
    server, local = make_bmi_classes(True)
    request, varname = make_var_request(local)

    raw_result = server.getVarLocation(request, None).location
    result = bmi_pb2.GetVarLocationResponse.Location.Name(raw_result).lower()

    expected = local.get_var_location(varname)
    assert result == expected
    del server


def test_get_var_value():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    setattr(request, "name", varname)
    values = local.get_value(varname, reserve_values(local, varname))
    numpy.testing.assert_allclose(
        numpy.reshape(server.getValue(request, reserve_values(local, varname)).values_double.values, values.shape),
        values)


def test_get_var_ptr():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    setattr(request, "name", varname)
    with pytest.raises(NotImplementedError):
        server.getValuePtr(request, None)


def test_get_value_at_indices():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    indices = numpy.array([29, 8, 19, 81])
    setattr(request, "name", varname)
    setattr(request, "indices", indices.flatten())
    setattr(request, "index_size", 1)
    dest = numpy.empty(4)
    values = local.get_value_at_indices(varname, dest, indices)
    numpy.testing.assert_allclose(server.getValueAtIndices(request, None).values_double.values, values.flatten())


def test_get_vals_indices_2d():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    indices = numpy.array([[0, 1], [1, 0], [2, 2]])
    setattr(request, "name", varname)
    setattr(request, "indices", indices.flatten())
    setattr(request, "index_size", 2)
    dest = numpy.empty(6)
    values = local.get_value_at_indices(varname, dest, indices)
    numpy.testing.assert_allclose(server.getValueAtIndices(request, None).values_double.values, values.flatten())


def test_set_var_values():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    dest = reserve_values(local, varname)
    values = 0.123 * local.get_value(varname, dest)
    setattr(request, "name", varname)
    setattr(request, "values_double", value_wrapper(values))
    setattr(request, "shape", values.shape)
    server.setValue(request, None)
    delattr(request, "values_double")
    delattr(request, "shape")
    response = server.getValue(request, None)
    numpy.testing.assert_allclose(values, response.values_double.values)


def test_set_values_indices():
    server, local = make_bmi_classes(True)
    request = RequestStub()
    varname = local.get_output_var_names()[0]
    indices = numpy.array([1, 11, 21])
    values = numpy.array([0.123, 4.567, 8.901])
    setattr(request, "name", varname)
    setattr(request, "values_double", value_wrapper(values))
    setattr(request, "indices", indices.flatten())
    setattr(request, "index_size", 1)
    server.setValueAtIndices(request, None)
    delattr(request, "values_double")
    response = server.getValueAtIndices(request, None)
    values_copy = numpy.array(response.values_double.values)
    numpy.testing.assert_allclose(values, values_copy)


def test_get_grid_size():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    assert server.getGridSize(request, None).size == local.get_grid_size(grid_id)


def test_get_grid_rank():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    assert server.getGridRank(request, None).rank == local.get_grid_rank(grid_id)


def test_get_grid_type():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    assert server.getGridType(request, None).type == local.get_grid_type(grid_id)


def test_get_grid_shape():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    result = server.getGridShape(request, reserve_grid_shape(local, grid_id)).shape
    expected = local.get_grid_shape(grid_id, reserve_grid_shape(local, grid_id))
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_spacing():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    result = server.getGridSpacing(request, reserve_grid_padding(local, grid_id)).spacing
    expected = local.get_grid_spacing(grid_id, reserve_grid_padding(local, grid_id))
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_origin():
    server, local = make_bmi_classes(True)
    grid_id, request = make_grid_request(local)
    result = server.getGridOrigin(request, reserve_grid_padding(local, grid_id)).origin
    expected = local.get_grid_origin(grid_id, reserve_grid_padding(local, grid_id))
    numpy.testing.assert_allclose(result, expected)


@pytest.mark.parametrize("server_method,server_request", [
    ('initialize', bmi_pb2.InitializeRequest(config_file='/data/config.ini')),
    ('update', bmi_pb2.Empty()),
    ('updateUntil', bmi_pb2.GetTimeResponse(time=42)),
    ('finalize', bmi_pb2.Empty()),
    ('getComponentName', bmi_pb2.Empty()),
    ('getInputItemCount', bmi_pb2.Empty()),
    ('getOutputItemCount', bmi_pb2.Empty()),
    ('getInputVarNames', bmi_pb2.Empty()),
    ('getOutputVarNames', bmi_pb2.Empty()),
    ('getTimeUnits', bmi_pb2.Empty()),
    ('getTimeStep', bmi_pb2.Empty()),
    ('getCurrentTime', bmi_pb2.Empty()),
    ('getStartTime', bmi_pb2.Empty()),
    ('getEndTime', bmi_pb2.Empty()),
    ('getVarGrid', bmi_pb2.GetVarRequest(name='something')),
    ('getVarType', bmi_pb2.GetVarRequest(name='something')),
    ('getVarItemSize', bmi_pb2.GetVarRequest(name='something')),
    ('getVarUnits', bmi_pb2.GetVarRequest(name='something')),
    ('getVarNBytes', bmi_pb2.GetVarRequest(name='something')),
    ('getValue', bmi_pb2.GetVarRequest(name='something')),
    ('getValueAtIndices', bmi_pb2.GetValueAtIndicesRequest(name='something', indices=(42,))),
    ('setValue', bmi_pb2.SetValueRequest(name='something', values_int=bmi_pb2.IntArrayMessage(values=(42,)))),
    ('setValueAtIndices', bmi_pb2.SetValueAtIndicesRequest(name='something', indices=(43,),
                                                           values_int=bmi_pb2.IntArrayMessage(values=(1234,)))),
    ('getGridSize', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridType', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridRank', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridShape', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridSpacing', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridOrigin', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridX', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridY', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridZ', bmi_pb2.GridRequest(grid_id=42)),

    ('getGridNodeCount', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridEdgeCount', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridFaceCount', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridEdgeNodes', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridFaceNodes', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridFaceEdges', bmi_pb2.GridRequest(grid_id=42)),
    ('getGridNodesPerFace', bmi_pb2.GridRequest(grid_id=42)),
])
def test_method_exceptions_with_stacktrace(server_method, server_request):
    exc = SomeException('Bmi method always fails')
    model = FailingModel(exc)
    server = BmiServer(model, True)
    context = Mock(grpc.ServicerContext)

    getattr(server, server_method)(server_request, context)

    context.abort_with_status.assert_called_once()
    status = context.abort_with_status.call_args[0][0]
    assert status.code == grpc.StatusCode.INTERNAL
    assert status.details == 'Bmi method always fails'
    metadata = status_pb2.Status.FromString(status.trailing_metadata[0][1])
    debuginfo = error_details_pb2.DebugInfo()
    metadata.details[0].Unpack(debuginfo)
    assert debuginfo.detail == "SomeException('Bmi method always fails',)"
    assert len(debuginfo.stack_entries) > 0


def test_get_grid_x():
    model = Rect3DGridModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridX(request, None).coordinates

    expected = numpy.array([0.1, 0.2, 0.3, 0.4])
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_y():
    model = Rect3DGridModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridY(request, None).coordinates

    expected = numpy.array([1.1, 1.2, 1.3])
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_z():
    model = Rect3DGridModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridZ(request, None).coordinates

    expected = numpy.array([2.1, 2.2])
    numpy.testing.assert_allclose(result, expected)


def test_grid_node_count():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridNodeCount(request, None).count

    assert result == 6


def test_grid_edge_count():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridEdgeCount(request, None).count

    assert result == 8


def test_grid_face_count():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridFaceCount(request, None).count

    assert result == 3


def test_get_grid_edge_nodes():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridEdgeNodes(request, None).edge_nodes

    expected = (0, 1, 1, 2, 2, 3, 3, 0, 1, 4, 4, 5, 5, 2, 5, 3)
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_face_nodes():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridFaceNodes(request, None).face_nodes

    expected = (0, 1, 2, 3, 1, 4, 5, 2, 2, 5, 3)
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_face_edges():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridFaceEdges(request, None).face_edges

    expected = (0, 1, 2, 3, 4, 5, 6, 1, 6, 7, 2)
    numpy.testing.assert_allclose(result, expected)


def test_get_grid_nodes_per_face():
    model = UnstructuredGridBmiModel()
    server = BmiServer(model, True)
    grid_id, request = make_grid_request(model)

    result = server.getGridNodesPerFace(request, None).nodes_per_face

    numpy.testing.assert_allclose(result, (4, 4, 3))



from bmipy import Bmi
import rpy2.robjects as robjects
import numpy as np


def build_model(class_name, source_fn=None):
    robjects.r('''
        build_model = function(class, source_fn=NULL) {
            if (!is.null(source_fn)) {
                source(source_fn)
            }
            return(get(class)$new())
        }
    ''')
    return robjects.r['build_model'](class_name, source_fn)


class BmiR(Bmi):
    """Python Wrapper of a R based sub class of bmi::AbstractBmi .

    AbstractBmi is available in https://github.com/eWaterCycle/bmi-r repo.

    Args:
        class_name (str): Name of R class which extends bmi::AbstractBmi
        source_fn (str): R file which contains `class_name` class
    """
    def __init__(self, class_name, source_fn=None):
        self.model = build_model(class_name, source_fn)

    # base
    def initialize(self, filename):
        self.model['bmi_initialize'](filename)

    def update(self):
        self.model['update']()

    def finalize(self):
        self.model['bmi_finalize']()

    # info
    def get_component_name(self):
        return self.model['getComponentName']()[0]

    def get_input_var_names(self):
        return self.model['getInputVarNames']()

    def get_output_var_names(self):
        return self.model['getOutputVarNames']()

    # time
    def get_start_time(self):
        return self.model['getStartTime']()[0]

    def get_current_time(self):
        return self.model['getCurrentTime']()[0]

    def get_end_time(self):
        return self.model['getEndTime']()[0]

    def get_time_step(self):
        return self.model['getTimeStep']()[0]

    def get_time_units(self):
        return self.model['getTimeUnits']()[0]

    # vars
    def get_var_type(self, name):
        return self.model['getVarType'](name)[0]

    def get_var_units(self, name):
        return self.model['getVarUnits'](name)[0]

    def get_var_itemsize(self, name):
        return self.model['getVarItemSize'](name)[0]

    def get_var_nbytes(self, name):
        return self.model['getVarNBytes'](name)[0]

    def get_var_grid(self, name):
        return self.model['getVarGrid'](name)[0]

    def get_var_location(self, name: str) -> str:
        return self.model['getVarLocation'](name)[0]

    # getter
    def get_value(self, name, dest):
        val = self.model['getValue'](name)
        np.copyto(src=val, dst=dest)
        return dest

    def get_value_at_indices(self, name, dest, indices):
        rindices = robjects.IntVector(indices)
        val = self.model['getValueAtIndices'](name, rindices)
        np.copyto(src=val, dst=dest)
        return dest

    def get_value_ptr(self, name):
        msg = 'Unable to implement get_value_ptr(), not possible to pass pointer between R and Python'
        raise NotImplementedError(msg)

    # setter
    def set_value(self, name, src):
        val = np.array(src)
        if val.dtype == np.int32:
            rsrc = robjects.IntVector(val)
        else:
            rsrc = robjects.FloatVector(val)
        self.model['setValue'](name, rsrc)

    def set_value_at_indices(self, name, indices, src):
        val = np.array(src)
        if val.dtype == np.int32:
            rsrc = robjects.IntVector(val)
        else:
            rsrc = robjects.FloatVector(val)
        rindices = robjects.IntVector(indices)
        self.model['setValueAtIndices'](name, rindices, rsrc)

    # grid
    def get_grid_rank(self, grid):
        return self.model['getGridRank'](grid)[0]

    def get_grid_size(self, grid):
        return self.model['getGridSize'](grid)[0]

    def get_grid_type(self, grid):
        return self.model['getGridType'](grid)[0]

    def get_grid_shape(self, grid, shape):
        val = self.model['getGridShape'](grid)
        np.copyto(src=val, dst=shape)
        return shape

    def get_grid_x(self, grid, x):
        val = np.array(self.model['getGridX'](grid))
        np.copyto(src=val, dst=x)
        return x

    def get_grid_y(self, grid, y):
        val = np.array(self.model['getGridY'](grid))
        np.copyto(src=val, dst=y)
        return y

    def get_grid_z(self, grid, z):
        val = np.array(self.model['getGridZ'](grid))
        np.copyto(src=val, dst=z)
        return z

    def get_grid_spacing(self, grid, spacing):
        val = np.array(self.model['getGridSpacing'](grid))
        np.copyto(src=val, dst=spacing)
        return spacing

    def get_grid_origin(self, grid, origin):
        val = np.array(self.model['getGridOrigin'](grid))
        np.copyto(src=val, dst=origin)
        return origin

    def get_grid_node_count(self, grid):
        return self.model['getGridNodeCount'](grid)[0]

    def get_grid_edge_count(self, grid):
        return self.model['getGridEdgeCount'](grid)[0]

    def get_grid_face_count(self, grid):
        return self.model['getGridFaceCount'](grid)[0]

    def get_grid_edge_nodes(self, grid, edge_nodes):
        result = self.model['getGridEdgeNodes'](grid)
        np.copyto(src=result, dst=edge_nodes)
        return edge_nodes

    def get_grid_face_nodes(self, grid, face_nodes):
        result = self.model['getGridFaceNodes'](grid)
        np.copyto(src=result, dst=face_nodes)
        return face_nodes

    def get_grid_nodes_per_face(self, grid, edge_nodes):
        result = self.model['getGridNodesPerFace'](grid)
        np.copyto(src=result, dst=edge_nodes)
        return edge_nodes



from typing import Tuple

import numpy
import numpy as np
from bmipy import Bmi

from grpc4bmi.utils import GRPC_MAX_MESSAGE_LENGTH


class SomeException(Exception):
    pass


class FailingModel(Bmi):
    def __init__(self, exc):
        self.exc = exc

    def initialize(self, filename):
        raise self.exc

    def update(self):
        raise self.exc

    def update_until(self, time: float) -> None:
        raise self.exc

    def finalize(self):
        raise self.exc

    def get_component_name(self):
        raise self.exc

    def get_input_item_count(self) -> int:
        raise self.exc

    def get_output_item_count(self) -> int:
        raise self.exc

    def get_input_var_names(self):
        raise self.exc

    def get_output_var_names(self):
        raise self.exc

    def get_start_time(self):
        raise self.exc

    def get_current_time(self):
        raise self.exc

    def get_end_time(self):
        raise self.exc

    def get_time_step(self):
        raise self.exc

    def get_time_units(self):
        raise self.exc

    def get_var_type(self, name):
        raise self.exc

    def get_var_units(self, name):
        raise self.exc

    def get_var_itemsize(self, name):
        raise self.exc

    def get_var_nbytes(self, name):
        raise self.exc

    def get_var_grid(self, name):
        raise self.exc

    def get_value(self, name, dest):
        raise self.exc

    def get_value_ptr(self, name):
        raise self.exc

    def get_value_at_indices(self, name, dest, inds):
        raise self.exc

    def set_value(self, name, src):
        raise self.exc

    def set_value_at_indices(self, name, inds, src):
        raise self.exc

    def get_grid_shape(self, grid, shape):
        raise self.exc

    def get_grid_x(self, grid, x):
        raise self.exc

    def get_grid_y(self, grid, y):
        raise self.exc

    def get_grid_z(self, grid, z):
        raise self.exc

    def get_grid_spacing(self, grid, spacing):
        raise self.exc

    def get_grid_origin(self, grid, origin):
        raise self.exc

    def get_grid_rank(self, grid):
        raise self.exc

    def get_grid_size(self, grid):
        raise self.exc

    def get_grid_type(self, grid):
        raise self.exc

    def get_var_location(self, name: str) -> str:
        raise self.exc

    def get_grid_node_count(self, grid: int) -> int:
        raise self.exc

    def get_grid_edge_count(self, grid: int) -> int:
        raise self.exc

    def get_grid_face_count(self, grid: int) -> int:
        raise self.exc

    def get_grid_edge_nodes(self, grid: int, edge_nodes: np.ndarray) -> np.ndarray:
        raise self.exc

    def get_grid_face_nodes(self, grid: int, face_nodes: np.ndarray) -> np.ndarray:
        raise self.exc

    def get_grid_nodes_per_face(self, grid: int, nodes_per_face: np.ndarray) -> np.ndarray:
        raise self.exc

    def get_grid_face_edges(self, grid: int, face_edges: np.ndarray) -> np.ndarray:
        raise self.exc


class GridModel(FailingModel):
    def __init__(self):
        super(GridModel, self).__init__(SomeException('not used'))

    def get_output_var_names(self) -> Tuple[str]:
        return 'plate_surface__temperature',

    def get_var_grid(self, name):
        return 0


class UniRectGridModel(GridModel):
    def get_grid_type(self, grid):
        return 'uniform_rectilinear'

    def get_grid_rank(self, grid):
        return 3

    def get_grid_size(self, grid):
        return 24

    def get_grid_shape(self, grid: int, shape: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[2, 3, 4], dst=shape)
        return shape

    def get_grid_origin(self, grid, dest):
        numpy.copyto(src=[0.1, 1.1, 2.1], dst=dest)
        return dest

    def get_grid_spacing(self, grid, dest):
        numpy.copyto(src=[0.1, 0.2, 0.3], dst=dest)
        return dest


class Rect3DGridModel(GridModel):
    def get_grid_type(self, grid):
        return 'rectilinear'

    def get_grid_size(self, grid):
        return 24

    def get_grid_rank(self, grid: int) -> int:
        return 3

    def get_grid_shape(self, grid: int, shape: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[2, 3, 4], dst=shape)
        return shape

    def get_grid_x(self, grid: int, x: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[0.1, 0.2, 0.3, 0.4], dst=x)
        return x

    def get_grid_y(self, grid: int, y: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[1.1, 1.2, 1.3], dst=y)
        return y

    def get_grid_z(self, grid: int, z: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[2.1, 2.2], dst=z)
        return z


class Rect2DGridModel(Rect3DGridModel):
    def get_grid_size(self, grid):
        return 12  # 4*3

    def get_grid_rank(self, grid: int) -> int:
        return 2

    def get_grid_shape(self, grid: int, shape: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[3, 4], dst=shape)
        return shape

    def get_grid_z(self, grid: int, z: np.ndarray) -> np.ndarray:
        raise NotImplementedError('Do not know what z is')


class Structured3DQuadrilateralsGridModel(GridModel):
    # Grid shape:
    #    0
    #   / \
    #  /   \
    # 3     1
    #  \    /
    #   \  /
    #    2
    #
    def get_grid_type(self, grid):
        return 'structured_quadrilateral'

    def get_grid_rank(self, grid: int) -> int:
        return 3

    def get_grid_size(self, grid):
        return 4

    def get_grid_shape(self, grid, shape):
        numpy.copyto(src=[1, 2, 2], dst=shape)
        return shape

    def get_grid_x(self, grid, x):
        numpy.copyto(src=[1.1, 0.1, 1.1, 2.1], dst=x)
        return x

    def get_grid_y(self, grid: int, y: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[2.2, 1.2, 0.2, 2.2], dst=y)
        return y

    def get_grid_z(self, grid: int, z: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[1.1, 2.2, 3.3, 4.4], dst=z)
        return z


class Structured2DQuadrilateralsGridModel(GridModel):
    # Grid shape:
    #    0
    #   / \
    #  /   \
    # 3     1
    #  \    /
    #   \  /
    #    2
    #
    def get_grid_type(self, grid):
        return 'structured_quadrilateral'

    def get_grid_rank(self, grid: int) -> int:
        return 2

    def get_grid_size(self, grid):
        return 4

    def get_grid_shape(self, grid, shape):
        numpy.copyto(src=[2, 2], dst=shape)
        return shape

    def get_grid_x(self, grid, x):
        numpy.copyto(src=[1.1, 0.1, 1.1, 2.1], dst=x)
        return x

    def get_grid_y(self, grid: int, y: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[2.2, 1.2, 0.2, 2.2], dst=y)
        return y

    def get_grid_z(self, grid: int, z: np.ndarray) -> np.ndarray:
        raise NotImplementedError('Do not know what z is')


class UnstructuredGridBmiModel(GridModel):
    # Uses grid example at https://bmi.readthedocs.io/en/latest/model_grids.html#unstructured-grids
    # Grid shape:
    #    3-----\
    #   / \     \
    # 0    \    \---5
    # \     2--/   /
    #  \    /     /
    #   \  /     /
    #    1---\  /
    #         4
    def get_grid_type(self, grid):
        return 'unstructured'

    def get_grid_shape(self, grid, dest):
        raise NotImplementedError('Do not know what shape is')

    def get_grid_size(self, grid):
        return 6

    def get_grid_rank(self, grid: int) -> int:
        return 2

    def get_grid_node_count(self, grid: int) -> int:
        return 6

    def get_grid_edge_count(self, grid: int) -> int:
        return 8

    def get_grid_face_count(self, grid: int) -> int:
        return 3

    def get_grid_edge_nodes(self, grid: int, edge_nodes: np.ndarray) -> np.ndarray:
        numpy.copyto(src=(0, 1, 1, 2, 2, 3, 3, 0, 1, 4, 4, 5, 5, 2, 5, 3), dst=edge_nodes)
        return edge_nodes

    def get_grid_face_nodes(self, grid: int, face_nodes: np.ndarray) -> np.ndarray:
        numpy.copyto(src=(0, 1, 2, 3, 1, 4, 5, 2, 2, 5, 3), dst=face_nodes)
        return face_nodes

    def get_grid_face_edges(self, grid: int, face_edges: np.ndarray) -> np.ndarray:
        numpy.copyto(src=(0, 1, 2, 3, 4, 5, 6, 1, 6, 7, 2), dst=face_edges)
        return face_edges

    def get_grid_nodes_per_face(self, grid: int, nodes_per_face: np.ndarray) -> np.ndarray:
        numpy.copyto(src=(4, 4, 3), dst=nodes_per_face)
        return nodes_per_face

    def get_grid_x(self, grid: int, x: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[0., 1., 2., 1., 3., 4.], dst=x)
        return x

    def get_grid_y(self, grid: int, y: np.ndarray) -> np.ndarray:
        numpy.copyto(src=[3., 1., 2., 4., 0., 3.], dst=y)
        return y

    def get_grid_z(self, grid: int, z: np.ndarray) -> np.ndarray:
        raise NotImplementedError('Do not know what z is')


class DTypeModel(GridModel):
    def __init__(self):
        super().__init__()
        self.dtype = numpy.dtype('float32')
        self.value = numpy.array((1.1, 2.2, 3.3), dtype=self.dtype)

    def get_var_type(self, name):
        return str(self.dtype)

    def get_var_itemsize(self, name):
        return self.dtype.itemsize

    def get_var_nbytes(self, name):
        return self.dtype.itemsize * self.value.size

    def get_value(self, name, dest):
        numpy.copyto(src=self.value, dst=dest)
        return dest

    def get_value_at_indices(self, name, dest, inds):
        numpy.copyto(src=self.value[inds], dst=dest)
        return dest

    def set_value(self, name, src):
        self.value[:] = src

    def set_value_at_indices(self, name, inds, src):
        self.value[inds] = src


class Float32Model(DTypeModel):
    pass


class Int32Model(DTypeModel):
    def __init__(self):
        super().__init__()
        self.dtype = numpy.dtype('int32')
        self.value = numpy.array((12, 24, 36), dtype=self.dtype)


class BooleanModel(DTypeModel):
    def __init__(self):
        super().__init__()
        self.dtype = numpy.dtype('bool')
        self.value = numpy.array((True, False, True), dtype=self.dtype)


class HugeModel(DTypeModel):
    """Model which has value which does not fit in message body

    Can be run from command line with

    ..code-block:: bash

        run-bmi-server --path $PWD/test --name fake_models.HugeModel --port 55555 --debug
    """
    def __init__(self):
        super().__init__()
        self.dtype = numpy.dtype('float64')
        # Create value which is bigger than 4Mb
        dimension = (3 * GRPC_MAX_MESSAGE_LENGTH) // self.dtype.itemsize + 1000
        self.value = numpy.ones((dimension,), dtype=self.dtype)



import logging
import math
import os
import socket
from contextlib import closing

import numpy as np
from bmipy import Bmi
import grpc
import numpy

from grpc_status import rpc_status
from google.rpc import error_details_pb2

from . import bmi_pb2, bmi_pb2_grpc
from .utils import GRPC_MAX_MESSAGE_LENGTH

log = logging.getLogger(__name__)


class RemoteException(grpc.RpcError):
    def __init__(self, message, tb):
        super().__init__(message)
        self.remote_stacktrace = tb


def handle_error(exc):
    """Parsers DebugInfo (https://github.com/googleapis/googleapis/blob/07244bb797ddd6e0c1c15b02b4467a9a5729299f/google/rpc/error_details.proto#L46-L52) from the trailing metadata of a grpc.RpcError

    Args:
        exc (grpc.RpcError): Exception to handle

    Raises: original exception or RemoteException

    """
    status = rpc_status.from_call(exc)
    if status is None:
        raise
    for detail in status.details:
        if detail.Is(error_details_pb2.DebugInfo.DESCRIPTOR):
            info = error_details_pb2.DebugInfo()
            detail.Unpack(info)
            remote_traceback = info.stack_entries
            remote_detail = info.detail
            raise RemoteException(remote_detail, remote_traceback) from exc
    raise


def _fits_in_message(array):
    """Tests whether array can be passed through a gRPC message with a max message size of 4Mb"""
    array_size = array.size * array.itemsize
    return array_size <= GRPC_MAX_MESSAGE_LENGTH


class BmiClient(Bmi):
    """
    Client BMI interface, implementing BMI by forwarding every function call via GRPC to the server connected to the
    same port. A GRPC channel can be passed to the constructor; if not, it constructs an insecure channel on a free
    port itself. The timeout parameter indicates the model BMI startup timeout parameter (s).

    >>> import grpc
    >>> from grpc4bmi.bmi_grpc_client import BmiClient
    >>> mymodel = BmiClient(grpc.insecure_channel("localhost:<PORT>"))
    >>> print(mymodel.get_component_name())
    Hello world
    """

    def __init__(self, channel=None, timeout=None, stub=None):
        if stub is None:
            c = BmiClient.create_grpc_channel() if channel is None else channel
            self.stub = bmi_pb2_grpc.BmiServiceStub(c)
            future = grpc.channel_ready_future(c)
            future.result(timeout=timeout)
        else:
            self.stub = stub

    def __del__(self):
        del self.stub

    @staticmethod
    def create_grpc_channel(port=0, host=None):
        p, h = port, host
        if h is None:
            h = "localhost"
        if p == 0:
            p = os.environ.get("BMI_PORT", 50051)
        return grpc.insecure_channel(':'.join([h, str(p)]))

    @staticmethod
    def get_unique_port(host=None):
        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
            s.bind(("" if host is None else host, 0))
            return int(s.getsockname()[1])

    def initialize(self, filename):
        fname = "" if filename is None else filename
        try:
            return self.stub.initialize(bmi_pb2.InitializeRequest(config_file=fname))
        except grpc.RpcError as e:
            handle_error(e)

    def update(self):
        try:
            self.stub.update(bmi_pb2.Empty())
        except grpc.RpcError as e:
            handle_error(e)

    def update_until(self, time: float) -> None:
        try:
            self.stub.updateUntil(bmi_pb2.GetTimeResponse(time=time))
        except grpc.RpcError as e:
            handle_error(e)

    def finalize(self):
        try:
            self.stub.finalize(bmi_pb2.Empty())
        except grpc.RpcError as e:
            handle_error(e)

    def get_component_name(self):
        try:
            return str(self.stub.getComponentName(bmi_pb2.Empty()).name)
        except grpc.RpcError as e:
            handle_error(e)

    def get_input_item_count(self) -> int:
        try:
            return self.stub.getInputItemCount(bmi_pb2.Empty()).count
        except grpc.RpcError as e:
            handle_error(e)

    def get_output_item_count(self) -> int:
        try:
            return self.stub.getOutputItemCount(bmi_pb2.Empty()).count
        except grpc.RpcError as e:
            handle_error(e)

    def get_input_var_names(self):
        try:
            return tuple([str(s) for s in self.stub.getInputVarNames(bmi_pb2.Empty()).names])
        except grpc.RpcError as e:
            handle_error(e)

    def get_output_var_names(self):
        try:
            return tuple([str(s) for s in self.stub.getOutputVarNames(bmi_pb2.Empty()).names])
        except grpc.RpcError as e:
            handle_error(e)

    def get_time_units(self):
        try:
            response = str(self.stub.getTimeUnits(bmi_pb2.Empty()).units)
            return None if not response else response
        except grpc.RpcError as e:
            handle_error(e)

    def get_time_step(self):
        try:
            return self.stub.getTimeStep(bmi_pb2.Empty()).interval
        except grpc.RpcError as e:
            handle_error(e)

    def get_current_time(self):
        try:
            return self.stub.getCurrentTime(bmi_pb2.Empty()).time
        except grpc.RpcError as e:
            handle_error(e)

    def get_start_time(self):
        try:
            return self.stub.getStartTime(bmi_pb2.Empty()).time
        except grpc.RpcError as e:
            handle_error(e)

    def get_end_time(self):
        try:
            return self.stub.getEndTime(bmi_pb2.Empty()).time
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_grid(self, name):
        try:
            return self.stub.getVarGrid(bmi_pb2.GetVarRequest(name=name)).grid_id
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_type(self, name):
        try:
            return str(self.stub.getVarType(bmi_pb2.GetVarRequest(name=name)).type)
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_itemsize(self, name):
        try:
            return self.stub.getVarItemSize(bmi_pb2.GetVarRequest(name=name)).size
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_units(self, name):
        try:
            response = str(self.stub.getVarUnits(bmi_pb2.GetVarRequest(name=name)).units)
            return None if not response else response
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_nbytes(self, name):
        try:
            return self.stub.getVarNBytes(bmi_pb2.GetVarRequest(name=name)).nbytes
        except grpc.RpcError as e:
            handle_error(e)

    def get_var_location(self, name: str) -> str:
        try:
            location = self.stub.getVarLocation(bmi_pb2.GetVarRequest(name=name)).location
            return bmi_pb2.GetVarLocationResponse.Location.Name(location).lower()
        except grpc.RpcError as e:
            handle_error(e)

    def get_value(self, name, dest):
        fits = _fits_in_message(dest)
        if not fits:
            return self._chunked_get_value(name, dest)
        try:
            response = self.stub.getValue(bmi_pb2.GetVarRequest(name=name))
            numpy.copyto(src=BmiClient.make_array(response), dst=dest)
            return dest
        except grpc.RpcError as e:
            handle_error(e)

    def _chunked_get_value(self, name: str, dest: np.array) -> np.array:
        # Make chunk one item smaller than maximum (4Mb)
        chunk_size = math.floor(GRPC_MAX_MESSAGE_LENGTH / dest.dtype.itemsize) - dest.dtype.itemsize
        chunks = []
        log.info(f'Too many items ({dest.size}) for single call, '
                 f'using multiple get_value_at_indices() with into chunks of {chunk_size} items')
        for i in range(0, dest.size, chunk_size):
            start = i
            stop = i + chunk_size
            # Last chunk can be smaller
            if stop > dest.size:
                stop = dest.size
            chunks.append(self._get_value_at_range(name, start, stop))

        numpy.concatenate(chunks, out=dest)
        return dest

    def _get_value_at_range(self, name, start, stop):
        log.info(f'Fetching value range {start} - {stop}')
        try:
            response = self.stub.getValueAtIndices(bmi_pb2.GetValueAtIndicesRequest(name=name,
                                                                                    indices=range(start, stop)))
            return BmiClient.make_array(response)
        except grpc.RpcError as e:
            handle_error(e)

    def get_value_ptr(self, name: str) -> np.ndarray:
        """Not possible, unable give reference to data structure in another process and possibly another machine"""
        raise NotImplementedError("Array references cannot be transmitted through this GRPC channel")

    def get_value_at_indices(self, name, dest, indices):
        try:
            index_array = indices
            if indices is list:
                index_array = numpy.array(indices)
            response = self.stub.getValueAtIndices(bmi_pb2.GetValueAtIndicesRequest(name=name,
                                                                                    indices=index_array.flatten()))
            numpy.copyto(src=BmiClient.make_array(response), dst=dest)
            return dest
        except grpc.RpcError as e:
            handle_error(e)

    def set_value(self, name, values):
        try:
            if values.dtype in (numpy.int16, numpy.int32, numpy.int64):
                request = bmi_pb2.SetValueRequest(name=name,
                                                  values_int=bmi_pb2.IntArrayMessage(values=values.flatten()))
            elif values.dtype in (numpy.float32, numpy.float16):
                request = bmi_pb2.SetValueRequest(name=name,
                                                  values_float=bmi_pb2.FloatArrayMessage(values=values.flatten()))
            elif values.dtype == numpy.float64:
                request = bmi_pb2.SetValueRequest(name=name,
                                                  values_double=bmi_pb2.DoubleArrayMessage(values=values.flatten()))
            else:
                raise NotImplementedError("Arrays with type %s cannot be transmitted through this GRPC channel" % values.dtype)
            self.stub.setValue(request)
        except grpc.RpcError as e:
            handle_error(e)

    def set_value_at_indices(self, name, inds, src):
        try:
            index_array = inds
            if inds is list:
                index_array = numpy.array(inds)
            if src.dtype in (numpy.int32, numpy.int64):
                request = bmi_pb2.SetValueAtIndicesRequest(name=name,
                                                           indices=index_array.flatten(),
                                                           values_int=bmi_pb2.IntArrayMessage(values=src.flatten()))
            elif src.dtype in (numpy.float32, numpy.float16):
                request = bmi_pb2.SetValueAtIndicesRequest(name=name,
                                                           indices=index_array.flatten(),
                                                           values_float=bmi_pb2.FloatArrayMessage(values=src.flatten()))
            elif src.dtype == numpy.float64:
                request = bmi_pb2.SetValueAtIndicesRequest(name=name,
                                                           indices=index_array.flatten(),
                                                           values_double=bmi_pb2.DoubleArrayMessage(values=src.flatten()))
            else:
                raise NotImplementedError("Arrays with type %s cannot be transmitted through this GRPC channel" % src.dtype)
            self.stub.setValueAtIndices(request)
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_size(self, grid):
        try:
            return self.stub.getGridSize(bmi_pb2.GridRequest(grid_id=grid)).size
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_rank(self, grid):
        try:
            return self.stub.getGridRank(bmi_pb2.GridRequest(grid_id=grid)).rank
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_type(self, grid):
        try:
            return str(self.stub.getGridType(bmi_pb2.GridRequest(grid_id=grid)).type)
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_x(self, grid, x):
        try:
            src = numpy.array(self.stub.getGridX(bmi_pb2.GridRequest(grid_id=grid)).coordinates)
            numpy.copyto(src=src, dst=x)
            return x
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_y(self, grid, y):
        try:
            src = numpy.array(self.stub.getGridY(bmi_pb2.GridRequest(grid_id=grid)).coordinates)
            numpy.copyto(src=src, dst=y)
            return y
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_z(self, grid, z):
        try:
            src = numpy.array(self.stub.getGridZ(bmi_pb2.GridRequest(grid_id=grid)).coordinates)
            numpy.copyto(src=src, dst=z)
            return z
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_shape(self, grid, shape):
        try:
            src = tuple(self.stub.getGridShape(bmi_pb2.GridRequest(grid_id=grid)).shape)
            numpy.copyto(src=src, dst=shape)
            return shape
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_spacing(self, grid, spacing):
        try:
            src = tuple(self.stub.getGridSpacing(bmi_pb2.GridRequest(grid_id=grid)).spacing)
            numpy.copyto(src=src, dst=spacing)
            return spacing
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_origin(self, grid, origin):
        try:
            src = tuple(self.stub.getGridOrigin(bmi_pb2.GridRequest(grid_id=grid)).origin)
            numpy.copyto(src=src, dst=origin)
            return origin
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_node_count(self, grid: int) -> int:
        try:
            return self.stub.getGridNodeCount(bmi_pb2.GridRequest(grid_id=grid)).count
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_edge_count(self, grid: int) -> int:
        try:
            return self.stub.getGridEdgeCount(bmi_pb2.GridRequest(grid_id=grid)).count
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_face_count(self, grid: int) -> int:
        try:
            return self.stub.getGridFaceCount(bmi_pb2.GridRequest(grid_id=grid)).count
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_edge_nodes(self, grid: int, edge_nodes: np.ndarray) -> np.ndarray:
        try:
            links = self.stub.getGridEdgeNodes(bmi_pb2.GridRequest(grid_id=grid)).edge_nodes
            numpy.copyto(src=links, dst=edge_nodes)
            return edge_nodes
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_face_nodes(self, grid: int, face_nodes: np.ndarray) -> np.ndarray:
        try:
            links = self.stub.getGridFaceNodes(bmi_pb2.GridRequest(grid_id=grid)).face_nodes
            numpy.copyto(src=links, dst=face_nodes)
            return face_nodes
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_face_edges(self, grid: int, face_edges: np.ndarray) -> np.ndarray:
        try:
            links = self.stub.getGridFaceEdges(bmi_pb2.GridRequest(grid_id=grid)).face_edges
            numpy.copyto(src=links, dst=face_edges)
            return face_edges
        except grpc.RpcError as e:
            handle_error(e)

    def get_grid_nodes_per_face(self, grid: int, nodes_per_face: np.ndarray) -> np.ndarray:
        try:
            links = self.stub.getGridNodesPerFace(bmi_pb2.GridRequest(grid_id=grid)).nodes_per_face
            numpy.copyto(src=links, dst=nodes_per_face)
            return nodes_per_face
        except grpc.RpcError as e:
            handle_error(e)

    @staticmethod
    def make_array(response):
        if response.HasField("values_int"):
            return numpy.array(response.values_int.values)
        if response.HasField("values_float"):
            return numpy.array(response.values_float.values)
        if response.HasField("values_double"):
            return numpy.array(response.values_double.values)



import numpy
from heat import BmiHeat


class FlatBmiHeat(BmiHeat):
    """The BmiHeat returns multi dimensional arrays, but grpc interface uses flat arrays

    """
    def get_value_at_indices(self, var_name, dest, indices):
        val = super(FlatBmiHeat, self).get_value_at_indices(var_name, indices).flatten()
        numpy.copyto(src=val, dst=dest)
        return dest

    def set_value(self, var_name, src):
        shape = super(FlatBmiHeat, self).get_grid_shape(super(FlatBmiHeat, self).get_var_grid(var_name))
        reshaped = numpy.reshape(src, shape)
        super(FlatBmiHeat, self).set_value(var_name, reshaped)

    def get_value(self, var_name, dest):
        val = super(FlatBmiHeat, self).get_value(var_name)
        numpy.copyto(src=val.flatten(), dst=dest)
        return dest

    def get_grid_shape(self, grid_id, dest=None):
        val = super(FlatBmiHeat, self).get_grid_shape(grid_id)
        if dest is None:
            return numpy.array(val)
        numpy.copyto(src=val, dst=dest)
        return dest

    def get_grid_spacing(self, grid_id, dest):
        val = super(FlatBmiHeat, self).get_grid_spacing(grid_id)
        numpy.copyto(src=val, dst=dest)
        return dest

    def get_grid_origin(self, grid_id, dest):
        val = super(FlatBmiHeat, self).get_grid_origin(grid_id)
        numpy.copyto(src=val, dst=dest)
        return dest


class FlatLegacyBmiHeat(BmiHeat):
    """BMI implementing interface defined in basic-modeling-interface 0.2

    https://pypi.org/project/basic-modeling-interface/0.2/

    """
    def set_value(self, var_name, src):
        shape = super(FlatLegacyBmiHeat, self).get_grid_shape(super(FlatLegacyBmiHeat, self).get_var_grid(var_name))
        reshaped = numpy.reshape(src, shape)
        super(FlatLegacyBmiHeat, self).set_value(var_name, reshaped)

    def get_value(self, var_name):
        val = super(FlatLegacyBmiHeat, self).get_value(var_name)
        return val.flatten()

    def get_value_at_indices(self, var_name, indices):
        return super(FlatLegacyBmiHeat, self).get_value_at_indices(var_name, indices).flatten()



"""Helpers to reserve numpy arrays for use in some of the Bmi methods as output argument
"""
import numpy

from bmipy import Bmi


def reserve_values(model: Bmi, name: str) -> numpy.ndarray:
    """Reserve dest for :func:`bmipy.Bmi.get_value`"""
    dtype = model.get_var_type(name)
    item_size = model.get_var_itemsize(name)
    total_size = model.get_var_nbytes(name)
    size = total_size // item_size
    return numpy.empty(size, dtype=dtype)


def reserve_grid_shape(model: Bmi, grid_id: int) -> numpy.ndarray:
    """Reserve shape for :func:`bmipy.Bmi.get_grid_shape`"""
    return numpy.empty(model.get_grid_rank(grid_id), dtype=numpy.int64)


def reserve_grid_padding(model: Bmi, grid_id: int) -> numpy.ndarray:
    """Reserve dest for :func:`bmipy.Bmi.get_grid_spacing` and :func:`bmipy.Bmi.get_grid_origin`
    """
    return numpy.empty(model.get_grid_rank(grid_id), dtype=numpy.float64)


def reserve_grid_nodes(model: Bmi, grid_id: int, dim_index: int) -> numpy.ndarray:
    """Reserve dest for :func:`bmipy.Bmi.get_grid_x`, :func:`bmipy.Bmi.get_grid_y` and :func:`bmipy.Bmi.get_grid_z`

    The dim_index goes x,y,z and model.get_grid_shape goes z,y,x or y,x so index is inverted
    """
    model_type = model.get_grid_type(grid_id)
    if model_type in {'uniform_rectilinear', 'rectilinear'}:
        grid_shape = model.get_grid_shape(grid_id, reserve_grid_shape(model, grid_id))
        shape = grid_shape[::-1][dim_index]
    else:
        shape = model.get_grid_size(grid_id)
    return numpy.empty(shape, dtype=numpy.float64)


def reserve_values_at_indices(model: Bmi, name: str, indices) -> numpy.ndarray:
    """Reserve dest for :func:`bmipy.Bmi.get_value_at_indices` """
    dtype = model.get_var_type(name)
    return numpy.empty(len(indices), dtype=dtype)
